@@ -62,7 +62,7 @@ int main() {
   FlowField q(Nx,Ny,Nz,1,Lx,Lz,a,b);
   u.addPerturbations(kxmax,kzmax,magnitude,decay);
 
-  FlowField omega(u); 
+  FlowField omega; 
 
   // Construct Navier-Stoke Integrator
   DNSFlags flags; 
@@ -90,18 +90,17 @@ int main() {
     }
 
     int nz0 = 12;
-    u.saveSliceXY(nz0,0,"uside");
-    u.saveSliceXY(nz0,1,"vside");
-    u.saveSliceXY(nz0,2,"wside");
-    u.saveSliceXY(nz0,2,"wside");
-    u.saveCrossSection(0,0,"usec");
-    u.saveCrossSection(0,1,"vsec");
-    u.saveCrossSection(0,2,"wsec");
+    u.saveSlice(2,0,nz0,"uside");
+    u.saveSlice(2,1,nz0,"vside");
+    u.saveSlice(2,2,nz0,"wside");
+    u.saveSlice(0,0,0,"usec");
+    u.saveSlice(0,1,0,"vsec");
+    u.saveSlice(0,2,0,"wsec");
 
-    u.vorticity2(omega);
-    omega.saveCrossSection(0,0,"omsec0");
-    omega.saveCrossSection(0,1,"omsec1");
-    omega.saveCrossSection(0,2,"omsec2");
+    curl(u, omega);
+    omega.saveSlice(0,0,0,"omsec0");
+    omega.saveSlice(0,1,0,"omsec1");
+    omega.saveSlice(0,2,0,"omsec2");
 
     // Take n steps of length dt
     dns.advance(u, q, dt.n());
