@@ -92,9 +92,9 @@ int main() {
 
   cout << "========================================================" << endl;
   cout << "Initial data, prior to time stepping" << endl;
-  cout << "L2Norm(un)             == " << L2Norm(un) << endl;
-  cout << "L2Norm(Pn)             == " << L2Norm(Pn) << endl;
-  cout << "div(un)           == " << un.divergence() << endl;  
+  cout << "L2Norm(un)    == " << L2Norm(un) << endl;
+  cout << "divNorm(un)   == " << divNorm(un) << endl;  
+  cout << "L2Norm(Pn)    == " << L2Norm(Pn) << endl;
   cout << "........................................................" << endl;
   cout << endl;
 
