@@ -377,18 +377,6 @@ Real BasisFunc::bcNorm(bool a_dirichlet, bool b_dirichlet) const {
   return sqrt(err);
 }
 
-Real BasisFunc::divNorm() const {
-  alwaysAssert(state_ == Spectral);
-  ComplexChebyCoeff div = u_;
-  div *= (2*pi*kx_/Lx_)*I;
-  ComplexChebyCoeff tmp = w_;
-  tmp *= (2*pi*kz_/Lz_)*I;
-  div += tmp;
-  diff(v_, tmp);
-  div += tmp;
-  return L2Norm(div);
-}
-
 bool BasisFunc::geometryCongruent(const BasisFunc& Phi) const {
   return (Ny_==Phi.Ny_ && Lx_==Phi.Lx_ && Lz_==Phi.Lz_ &&
 	  a_==Phi.a_ && b_==Phi.b_);
@@ -722,6 +710,77 @@ Complex innerProduct(const BasisFunc& f, const BasisFunc& g, NormType n,
   return (n==L2NORM) ? L2InnerProduct(f,g,nrmlz) : chebyInnerProduct(f,g,nrmlz);
 }
 
+// =================================================================
+// boundary and divergence norms
+
+Real divNorm2(const BasisFunc& f, bool normalize) {
+  assert(f.state() == Spectral);
+  ComplexChebyCoeff div = f.u();
+  div *= (2*pi*f.kx()/f.Lx())*I;
+  ComplexChebyCoeff tmp = f.w();
+  tmp *= (2*pi*f.kz()/f.Lz())*I;
+  div += tmp;
+  diff(f.v(), tmp);
+  div += tmp;
+  return L2Norm2(div, normalize);
+}
+
+Real divNorm(const BasisFunc& f, bool normalize)  {
+  return sqrt(divNorm2(f, normalize));
+}
+
+Real divDist2(const BasisFunc& f, const BasisFunc& g, bool normalize) {
+  assert(f.state() == Spectral && g.state() == Spectral);
+  assert(f.congruent(g));
+  ComplexChebyCoeff div = f.u();
+  div -= g.u();
+  div *= (2*pi*f.kx()/f.Lx())*I;
+  ComplexChebyCoeff tmp = f.w();
+  tmp -= g.w();
+  tmp *= (2*pi*f.kz()/f.Lz())*I;
+  div += tmp;
+  diff(f.v(), tmp);
+  div += tmp;
+  diff(g.v(), tmp);
+  div -= tmp;
+  return L2Norm2(div, normalize);
+}
+
+Real divDist(const BasisFunc& f, const BasisFunc& g, bool normalize)  {
+  return sqrt(divDist2(f,g, normalize));
+}
+
+Real bcNorm2(const BasisFunc& f, bool normalize) {
+  Real bc2 = 
+    norm2(f.u().eval_a()) + norm2(f.u().eval_b()) +
+    norm2(f.v().eval_a()) + norm2(f.v().eval_b()) +
+    norm2(f.w().eval_a()) + norm2(f.w().eval_b());
+  if (!normalize)
+    bc2 *= f.Lx()*f.Lz();
+  return bc2;
+}
+
+Real bcNorm(const BasisFunc& f, bool normalize)  {
+  return sqrt(bcNorm2(f, normalize));
+}
+
+Real bcDist2(const BasisFunc& f, const BasisFunc& g, bool normalize) {
+  Real bc2 = 
+    norm2(f.u().eval_a() - g.u().eval_a()) + 
+    norm2(f.u().eval_b() - g.u().eval_b()) + 
+    norm2(f.v().eval_a() - g.v().eval_a()) + 
+    norm2(f.v().eval_b() - g.v().eval_b()) + 
+    norm2(f.w().eval_a() - g.w().eval_a()) + 
+    norm2(f.w().eval_b() - g.w().eval_b());
+  if (!normalize)
+    bc2 *= f.Lx()*f.Lz();
+  return bc2;
+}
+
+Real bcNorm(const BasisFunc& f, const BasisFunc& g, bool normalize)  {
+  return sqrt(bcNorm2(f, normalize));
+}
+
 BasisFunc xdiff(const BasisFunc& phi) {
   BasisFunc phix(phi);
   phix *= (2*pi*phi.kx()/phi.Lx())*I;
