@@ -542,6 +542,28 @@ bool operator==(const ChebyCoeff& u, const ChebyCoeff& v) {
   return true;
 }
 
+void swap(ChebyCoeff& f, ChebyCoeff& g) {
+  Real rtmp = f.a_;
+  f.a_ = g.a_;
+  g.a_ = rtmp;
+  rtmp = f.b_;
+  f.b_ = g.b_;
+  g.b_ = rtmp;
+
+  fieldstate stmp = f.state_;
+  f.state_ = g.state_;
+  g.state_ = stmp;
+
+  int itmp = f.N_;
+  f.N_ = g.N_;
+  g.N_ = itmp;
+
+  Real* dtmp = f.data_;
+  f.data_ = g.data_;
+  g.data_ = dtmp;
+}
+    
+
 void integrate(const ChebyCoeff& dudy, ChebyCoeff& u) {
   assert(dudy.state() == Spectral);
   int N=dudy.numModes();
@@ -614,6 +636,18 @@ void diff2(const ChebyCoeff& u, ChebyCoeff& d2udy2, ChebyCoeff& tmp) {
   diff(tmp, d2udy2);
 }
 
+void diff(const ChebyCoeff& f, ChebyCoeff& df, int n) {
+  assert(n>=0);
+  assert(f.state() == Spectral);
+  df = f;
+  ChebyCoeff tmp;
+  for (int k=0; k<n; ++k) {
+    diff(df,tmp);
+    swap(df,tmp);
+  }
+  return;
+}
+
 ChebyCoeff diff(const ChebyCoeff& u) {
   ChebyCoeff du(u.length(), u.a(), u.b(), Spectral);
   diff(u, du);
@@ -626,6 +660,18 @@ ChebyCoeff diff2(const ChebyCoeff& u) {
   return du2;
 }
 
+ChebyCoeff diff(const ChebyCoeff& f, int n) {
+  assert(n>=0);
+  ChebyCoeff  g = f;
+  ChebyCoeff gy(f.N(), f.a(), f.b(), Spectral);
+
+  for (int n_=0; n_<n; ++n_) {
+    diff(g,gy);
+    swap(g,gy);
+  }
+  return g;
+}
+
 Vector chebypoints(int N, Real a, Real b) {
   Vector xcheb(N);
   Real piN = pi/(N-1);
@@ -1017,6 +1063,12 @@ bool operator==(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v) {
   return u.re == v.re && u.im == v.im;
 }
 
+void swap(ComplexChebyCoeff& f, ComplexChebyCoeff& g) {
+  swap(f.re,g.re);
+  swap(f.im,g.im);
+}
+    
+
 ComplexChebyCoeff operator*(Real c, const ComplexChebyCoeff& v) {
   ComplexChebyCoeff rtn(v);
   rtn *= c;
@@ -1049,6 +1101,10 @@ void diff2(const ComplexChebyCoeff& u, ComplexChebyCoeff& d2u, ComplexChebyCoeff
   diff2(u.im, d2u.im, tmp.im);
 }
 
+void diff(const ComplexChebyCoeff& u, ComplexChebyCoeff& du, int n) {
+  diff(u.re, du.re, n);
+  diff(u.im, du.im, n);
+}
 ComplexChebyCoeff diff(const ComplexChebyCoeff& u) {
   ComplexChebyCoeff du(u.numModes(), u.a(), u.b(), Spectral);
   diff(u.re, du.re);
@@ -1061,6 +1117,12 @@ ComplexChebyCoeff diff2(const ComplexChebyCoeff& u) {
   diff2(u.im, du2.im);
   return du2;
 }
+ComplexChebyCoeff diff(const ComplexChebyCoeff& u, int n) {
+  ComplexChebyCoeff du(u.numModes(), u.a(), u.b(), Spectral);
+  diff(u.re, du.re, n);
+  diff(u.im, du.im, n);
+  return du;
+}
 
 void integrate(const ComplexChebyCoeff& du, ComplexChebyCoeff& u) {
   integrate(du.re, u.re);
