@@ -1,7 +1,7 @@
 /* nsintegrator.cpp: time-integration class for spectral Navier-Stokes DNS
  * Channelflow-0.9
  *
- * Copyright (C) 2001  John F. Gibson  
+ * Copyright (C) 2001-2005  John F. Gibson  
  *  
  * jgibson@mail.sjcsf.edu  
  * John F. Gibson 
@@ -107,7 +107,6 @@ NSIntegrator::NSIntegrator()
   //gamma_(0),
   //zeta_(0),
   flags_(), 
-  alternator_(false),
   dPdxRef_(0),
   dPdxAct_(0),
   UbulkRef_(0), 
@@ -159,7 +158,6 @@ NSIntegrator::NSIntegrator(FlowField& u, const ChebyCoeff& Ubase,
   //gamma_(Nsubsteps_),
   //zeta_(Nsubsteps_),
   flags_(flags),
-  alternator_(false),
   dPdxRef_(0),
   dPdxAct_(0),
   UbulkRef_(0), 
@@ -246,34 +244,17 @@ NSIntegrator::NSIntegrator(FlowField& u, const ChebyCoeff& Ubase,
     t.makePhysical(Ubasey_);
     // keep Ubaseyy_ as spectral for calc of R in advance() method..
   }
-  
-  // Calculate nonlin term and vorticity in order to get initial CFL number.
-  switch (flags_.nonlinearity) {
-  case SkewSymmetric:
-    // SkewSymmetric requires a 9d (3x3) tmp flowfield
+
+  // Alternating and SkewSymmetric require a 9d (3x3) tmp flowfield
+  if (flags_.nonlinearity ==  Alternating || 
+      flags_.nonlinearity ==  _Alternating || 
+      flags_.nonlinearity ==  Convection || 
+      flags_.nonlinearity ==  Divergence || 
+      flags_.nonlinearity ==  SkewSymmetric) 
     vortn_.resize(u.Nx(), u.Ny(), u.Nz(), 9, u.Lx(), u.Lz(), u.a(), u.b());
-    u.nonlinearitySkewSymmetric(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
-    break;
-  case Alternating:
-    // SkewSymmetric requires a 9d (3x3) tmp flowfield
-    vortn_.resize(u.Nx(), u.Ny(), u.Nz(), 9, u.Lx(), u.Lz(), u.a(), u.b());
-    if (alternator_) 
-      u.nonlinearityConvection(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
-    else 
-      u.nonlinearityDivergence(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
-    alternator_ = !alternator_;
-    break;
-  case Linearized:
-    u.nonlinearityLinearized(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
-    break;
-  case Rotational:
-  default:
-    if (flags_.dealias_y())
-      u.nonlinearityDealiased(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
-    else
-      u.nonlinearityRotational(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
-  }
-    
+
+  navierstokesNL(u, Ubase_, fn_, vortn_, flags_.nonlinearity);
+
   cfl_ = u.CFLfactor();
   cfl_ *= flags_.dealias_xz() ? 2.0*pi/3.0*dt_ : pi*dt_;
 
@@ -337,7 +318,6 @@ NSIntegrator& NSIntegrator::operator=(const NSIntegrator& ns) {
   //gamma_ = ns.gamma_;
   //zeta_ = ns.zeta_;
   flags_ = ns.flags_;
-  alternator_ = ns.alternator_;
   dPdxRef_ = ns.dPdxRef_;
   dPdxAct_ = ns.dPdxAct_;
   UbulkRef_ = ns.UbulkRef_;
@@ -410,7 +390,7 @@ void NSIntegrator::reset(Real nu, Real dt) {
 	// with momentum eqns. Don't bother to initialize TauSolvers for these.
 	if (!flags_.dealias_xz() || !isAliasedMode(kx,kz)) 
 	  tausolver_[substep][nx][nz] = TauSolver(kx, kz, Lx_, Lz_, a_, b_, 
-						  lambda, nu, Nyd_, true);
+						  lambda, nu, Nyd_, flags_.taucorrection);
       }
     }
   }
@@ -466,9 +446,9 @@ void NSIntegrator::debugfunc(const FlowField& u) const {
   cout << "L2Norm2(v)   == " << L2Norm2(v) << endl;
   cout << "L2Norm2(u-v) == " << L2Norm2(w) << endl;
   cout << "L2Dist2(u,v) == " << L2Dist2(u,v) << endl;
-  cout << "    div(u)   == " << u.divergence() << endl;
-  cout << "    div(v)   == " << v.divergence() << endl;
-  cout << "    div(u-v) == " << w.divergence() << endl;
+  cout << "    div(u)   == " << divNorm2(u) << endl;
+  cout << "    div(v)   == " << divNorm2(v) << endl;
+  cout << "    div(u-v) == " << divDist2(u,v) << endl;
 
   u.saveSpectrum("u");
   v.saveSpectrum("v");
@@ -479,8 +459,8 @@ void NSIntegrator::debugfunc(const FlowField& u) const {
   w.saveDivSpectrum("wd");
   cfpause();
 }
-  
 
+// This calculation is outlined in 4/09/01 thesis notes and Channelflow Manual
 void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
 
   bool verify = (flags_.verbosity == VerifyTauSolve || 
@@ -499,35 +479,20 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
       // Update nonlinear terms fn and fn1. Swap() switches pointers to the 
       // data inside the objects, so saving a copy. 
       swap(fn1_,fn_);
-
-      switch (flags_.nonlinearity) {
-      case SkewSymmetric:
-	un.nonlinearitySkewSymmetric(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
-	break;
-      case Alternating:
-	if (alternator_) 
-	  un.nonlinearityConvection(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
-	else 
-	  un.nonlinearityDivergence(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
-	break;
-      case Linearized:
-	un.nonlinearityLinearized(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
-	break;
-      case Rotational:
-      default:
-	if (flags_.dealias_y())
-	  un.nonlinearityDealiased(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
-	else
-	  un.nonlinearityRotational(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
-      }
+      navierstokesNL(un, Ubase_, fn_, vortn_, flags_.nonlinearity);
+      //calculate_nonlinearity(un, vortn_, fn_);
+      
       //cout << "After nonlinearity calc..." << endl;
       //debugfunc(un);
 
       //cerr << "NSI: SUBSTEP==" << substep << "L2Norm(fn) == " << L2Norm(fn_) << endl;
-      Real ab = alpha_[substep]/beta_[substep]; 
-      Real ngb = -gamma_[substep]/beta_[substep]; 
-      Real nzb = -zeta_[substep]/beta_[substep];  
-      Real abnu = ab*nu_;           
+
+      // Set convenience variables for CNAB2 or RK3 time-stepping coefficients
+      Real a_b = alpha_[substep]/beta_[substep]; 
+      Real ng_b = -gamma_[substep]/beta_[substep]; 
+      Real nz_b = -zeta_[substep]/beta_[substep];  
+      Real anu_b = a_b*nu_;           
+      Real anu = alpha_[substep]*nu_;           
 
       // Update with the time-stepping algorthim
       // Each (kx,kz) pair is an independent calculation, so parallelism
@@ -538,6 +503,8 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
 	const int kx = un.kx(nx);
 	for (int nz=0; nz<Nz_; ++nz) {
 	  const int kz = un.kz(nz);
+
+	  // Zero out the aliased modes
 	  if ((kx==kxmax || kz==kzmax) ||
 	      (flags_.dealias_xz() && isAliasedMode(kx,kz))) {
 	    for (int ny=0; ny<Nyd_; ++ny) {
@@ -548,46 +515,52 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
 	    }
 	    break;
 	  }
-	  // Put alpha/beta nu u_n into (uk,vk,wk),
-	  //  -(gamma/beta fn + zeta/beta fn1) into (fxk,fyk,fzk),
-	  // and alpha/beta qn into Pk.
-	  int ny; // MSVC++ FOR-SCOPE BUG
-	  for (ny=0; ny<Nyd_; ++ny) {
-	    uk_.set(ny, abnu*un.cmplx(nx,ny,nz,0));
-	    vk_.set(ny, abnu*un.cmplx(nx,ny,nz,1));
-	    wk_.set(ny, abnu*un.cmplx(nx,ny,nz,2));
-	    Pk_.set(ny, ab*qn.cmplx(nx,ny,nz,0));
-	    fxk_.set(ny, ngb*( fn_.cmplx(nx,ny,nz,0)) 
-		     +   nzb*(fn1_.cmplx(nx,ny,nz,0)));
-	    fyk_.set(ny, ngb*( fn_.cmplx(nx,ny,nz,1)) 
-		     +   nzb*(fn1_.cmplx(nx,ny,nz,1)));
-	    fzk_.set(ny, ngb*( fn_.cmplx(nx,ny,nz,2)) 
-		     +   nzb*(fn1_.cmplx(nx,ny,nz,2)));
+
+	  // Make the following assignments in prep for computation of RHS
+	  //    (uk,vk,wk) = alpha/beta nu u_n 
+	  //            Pk = alpha/beta qn 
+	  // (fxk,fyk,fzk) = (gamma/beta fn + zeta/beta fn1) 
+	  for (int ny=0; ny<Nyd_; ++ny) {
+	    uk_.set(ny, anu_b*un.cmplx(nx,ny,nz,0));
+	    vk_.set(ny, anu_b*un.cmplx(nx,ny,nz,1));
+	    wk_.set(ny, anu_b*un.cmplx(nx,ny,nz,2));
+	    Pk_.set(ny, a_b*qn.cmplx(nx,ny,nz,0));
+	    fxk_.set(ny, ng_b*( fn_.cmplx(nx,ny,nz,0)) 
+		     +   nz_b*(fn1_.cmplx(nx,ny,nz,0)));
+	    fyk_.set(ny, ng_b*( fn_.cmplx(nx,ny,nz,1)) 
+		     +   nz_b*(fn1_.cmplx(nx,ny,nz,1)));
+	    fzk_.set(ny, ng_b*( fn_.cmplx(nx,ny,nz,2)) 
+		     +   nz_b*(fn1_.cmplx(nx,ny,nz,2)));
 	  }
 	  
-	  // Put abnu un" into in Rx, etc., using Pyk_ as a tmp.
+	  // Start accumulating terms of RHS into R = (Rxk,Ryk,Rzk). Result is
+	  // R = a/b nu un" + [1/(b dt)- a/b nu kappa2] un - a/b grad qn 
+	  //     - g/b fn - z/b fn1
+
+	  // (1) Put a/b nu un" into in R. (diff2 uses Pyk_ as tmp workspace)
 	  diff2(uk_, Rxk_, Pyk_);
 	  diff2(vk_, Ryk_, Pyk_);
 	  diff2(wk_, Rzk_, Pyk_);
 
-	  // Compute y-comp of pressure gradient
+	  // (2) Put a/b qn' into Pyk (compute y-comp of pressure gradient).
 	  diff(Pk_, Pyk_); 
       
-	  // Add remainder of RHS terms to R. After this step,
-	  // R = a/b nu un" + [1/(b dt)- a/b nu kappa2] un - a/b grad P 
-	  //       - g/b fn - z/b fn1
-
-	  // At this point the variable uk_ == nu a/b un. So
-	  //    [1/(a nu dt) - kappa2] * uk 
-	  // == [1/(a nu dt) - kappa2] * nu a/b un 
-	  // == [1/(beta dt) - nu a/b kappa2] un 
-	  // which is what 4/09/01 notes call for in next term
-
+	  // (3) Add 
+	  //     [1/(b dt)- a/b nu kappa2] un - a/b grad qn - g/b fn - z/b fn1
+	  // to R.
+	  //
+	  // Note: At this point the variable uk_ == nu a/b un. So the 
+	  // [1/(b dt)- a/b nu kappa2] un term is added into R by adding
+	  // [1/(a nu dt) - kappa2] uk  == [1/(a nu dt) - kappa2] nu a/b un 
+	  //                            == [1/(b dt)    - nu a/b kappa2] un 
+	  //
+	  // Similarly, at this point Pk == nu a/b qn and Pyk = nu a/b qn'.
+	  // and fk = -g/b fn - z/b fn1
 	  Real kappa2 = 4*pi*pi*(square(kx/Lx_) + square(kz/Lz_));
-	  Real c = 1.0/(alpha_[substep]*nu_*dt_) - kappa2;
+	  Real c = 1.0/(anu*dt_) - kappa2;
 	  const Complex pi2IkxLx = I*(2*pi*kx/Lx_);
 	  const Complex pi2IkzLz = I*(2*pi*kz/Lz_);
-	  for (ny=0; ny<Nyd_; ++ny) {
+	  for (int ny=0; ny<Nyd_; ++ny) {
 	    Rxk_.add(ny, c*uk_[ny] + fxk_[ny] - pi2IkxLx*Pk_[ny]);
 	    Ryk_.add(ny, c*vk_[ny] + fyk_[ny] - Pyk_[ny]);
 	    Rzk_.add(ny, c*wk_[ny] + fzk_[ny] - pi2IkzLz*Pk_[ny]);
@@ -599,7 +572,7 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
 
 	    // The x-comp for R has an additional term from base flow
 	    // curvature, in both cases of const mass flux and const press grad
-	    Real c = nu_*(ab+1.0);
+	    Real c = nu_*(a_b+1.0);
 	    if (Ubaseyy_.length() > 0)
 	      for (int ny=0; ny<Ny_; ++ny)
 		Rxk_.re[ny] += c*Ubaseyy_[ny];
@@ -610,11 +583,15 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
 	      // gradient. Distinguish between dPdxAct and dPdxRef for the 
 	      // rare case that dPdxRef has just been changed externally. 
 	      // Thesis notes 4/09/01.
-	      Rxk_.re[0] -= ab*dPdxAct_ + dPdxRef_;
+	      Rxk_.re[0] -= a_b*dPdxAct_ + dPdxRef_;
 
 	      tausolver_[substep][nx][nz].solve(uk_,vk_,wk_,Pk_, Rxk_,Ryk_,Rzk_);
-	      if (verify) 
-		tausolver_[substep][nx][nz].verify(uk_,vk_,wk_,Pk_, Rxk_,Ryk_,Rzk_);
+	      if (verify) {
+		Real error = 
+		  tausolver_[substep][nx][nz].verify(uk_,vk_,wk_,Pk_, Rxk_,Ryk_,Rzk_);		
+		cout << "Tau solve error == " << error << endl;
+	      }
+
 	      UbulkAct_ = UbulkBase_ + uk_.re.mean();
 	      dPdxAct_ = dPdxRef_;
 	    }
@@ -622,7 +599,7 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
 	      // CONST MASS FLUX
 	      // Add the previous time-step's dPdx to the RHS. The next 
 	      // timestep's dPdx term appears on LHS as unknown.
-	      Rxk_.re[0] -= ab*dPdxAct_;
+	      Rxk_.re[0] -= a_b*dPdxAct_;
 
 	      // Use tausolver with additional variable and constraint:
 	      // free variable: dPdxAct at next time-step, 
@@ -630,11 +607,13 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
 	      tausolver_[substep][nx][nz].solve(uk_, vk_, wk_, Pk_, dPdxAct_, 
 						Rxk_, Ryk_, Rzk_, 
 						UbulkRef_ - UbulkBase_);
-	      if (verify)
+	      if (verify) {
+		Real error =
 		tausolver_[substep][nx][nz].verify(uk_, vk_, wk_, Pk_, 
 						   dPdxAct_,  Rxk_, Ryk_, 
 						   Rzk_, UbulkRef_-UbulkBase_);
-		
+		cout << "Tau solve error == " << error << endl;
+	      }
 
 	      
 	      UbulkAct_ = UbulkBase_ + uk_.re.mean(); // should == UbulkRef_
@@ -644,22 +623,53 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
 	      //cout << "       dPdx == " << dPdx_ << endl;
 	      //cout << "UbulkRef_ - UbulkAct_ == " << UbulkDiff << endl;
 	    }
+	    // for kx=kz=0, constant term of pressure is arbitrary 3/19/05
+	    //Pk_.set(0, Complex(0.0, 0.0));
 	  }
 	  // for all cases but kx==kz==0, both const press grad and const flux.
 	  else {
 	    tausolver_[substep][nx][nz].solve(uk_,vk_,wk_,Pk_, Rxk_,Ryk_,Rzk_);
-	    if (verify)
-	      tausolver_[substep][nx][nz].verify(uk_,vk_,wk_,Pk_, Rxk_,Ryk_,Rzk_,true);
+	    if (verify) {
+	      Real error = tausolver_[substep][nx][nz].verify(uk_,vk_,wk_,Pk_, Rxk_,Ryk_,Rzk_,true);
+	      cout << "Tau solve error == " << error << endl;
+
+	      if (error > 1e-7) {
+		cout << "NSIntegrator::advance(...): Large tau-solve error" <<endl;
+		cout << "kx,kz, substep == " << kx << ',' << kz << ", " << substep << endl;
+		uk_.makePhysical();
+		vk_.makePhysical();
+		wk_.makePhysical();
+		Pk_.makePhysical();
+		Rxk_.makePhysical();
+		Ryk_.makePhysical();
+		Rzk_.makePhysical();
+		uk_.save("uk");
+		vk_.save("vk");
+		wk_.save("wk");
+		Pk_.save("Pk");
+		Rxk_.save("Rxk");
+		Ryk_.save("Ryk");
+		Rzk_.save("Rzk");
+		uk_.makeSpectral();
+		vk_.makeSpectral();
+		wk_.makeSpectral();
+		Pk_.makeSpectral();
+		Rxk_.makeSpectral();
+		Ryk_.makeSpectral();
+		Rzk_.makeSpectral();
+		cfpause();
+	      }
+	    }
 	  }
 	  // Load solutions back into the full 3d data arrays.
-	  for (ny=0; ny<Nyd_; ++ny) {
+	  for (int ny=0; ny<Nyd_; ++ny) {
 	    un.cmplx(nx,ny,nz,0) = uk_[ny];
 	    un.cmplx(nx,ny,nz,1) = vk_[ny];
 	    un.cmplx(nx,ny,nz,2) = wk_[ny];
 	    qn.cmplx(nx,ny,nz,0) = Pk_[ny];
 	  }
 	  // Load solutions back into the full 3d data arrays.
-	  for (ny=Nyd_; ny<Ny_; ++ny) {
+	  for (int ny=Nyd_; ny<Ny_; ++ny) {
 	    un.cmplx(nx,ny,nz,0) = 0.0;
 	    un.cmplx(nx,ny,nz,1) = 0.0;
 	    un.cmplx(nx,ny,nz,2) = 0.0;
@@ -676,7 +686,6 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
       //cout.form("%4.2f ", t_);
       cout.flush();
     }
-    alternator_ = !alternator_;  // swap Convection/Divergence if Alternating
   }
   //cout << "After integration..." << endl;
   //debugfunc(un);
@@ -748,89 +757,6 @@ void assignOrrSommNonlin(FlowField& nonlin, Real t, Complex omega, const BasisFu
   nonlin.chebyfft_y();
 }
 
-void Uuq2p(const ChebyCoeff& Ubase, const FlowField& un, const FlowField& q, 
-	   FlowField& p, bool linearized) {
-  
-
-  if (linearized) {
-    p = q;
-    return;
-  }
-  ChebyCoeff U(Ubase);
-  ChebyTransform trans(Ubase.numModes());
-  U.makePhysical(trans);
-  
-  int Nx=un.numXgridpts();
-  int Ny=un.numYgridpts();
-  int Nz=un.numZgridpts();
-
-  p = q; 
-  p.makePhysical();
-
-  FlowField& u = (FlowField&) un;
-  fieldstate uxzstate = u.xzstate();
-  fieldstate uystate = u.ystate();
-  u.makePhysical();
-
-  // At this point 
-  // u == utot - U
-  // p == q == ptrue + 1/2 (utot dot utot)
-  
-  // Remove 1/2 utot dot utot from p
-  for (int ny=0; ny<Ny; ++ny) {
-    //Real Uny = Ubase[ny];
-    for (int nx=0; nx<Nx; ++nx)
-      for (int nz=0; nz<Nz; ++nz) 
-	p(nx,ny,nz,0) -= 0.5*(square(u(nx,ny,nz,0)) + //  + Uny) + 
-			      square(u(nx,ny,nz,1)) + 
-			      square(u(nx,ny,nz,2))); 
-  }
-  u.makeState(uxzstate, uystate); 
-  p.makeState(q.xzstate(), q.ystate()); 
-}
-
-void Uup2q(const ChebyCoeff& Ubase, const FlowField& un, const FlowField& p, 
-	   FlowField& q, bool linearized) {
-
-  if (linearized) {
-    q=p;
-    return;
-  }
-
-
-  ChebyCoeff U(Ubase);
-  ChebyTransform trans(Ubase.numModes());
-  U.makePhysical(trans);
- 
-  int Nx=un.numXgridpts();
-  int Ny=un.numYgridpts();
-  int Nz=un.numZgridpts();
-  
-  q = p;
-  q.makePhysical();
-
-  FlowField& u = (FlowField&) un;
-  fieldstate uxzstate = u.xzstate();
-  fieldstate uystate = u.ystate();
-
-  u.makePhysical();
-  // At this point 
-  // u == utot - U
-  // q == p
-  
-  // Add 1/2 utot dot utot from p
-  for (int ny=0; ny<Ny; ++ny) {
-    //Real Uny = Ubase[ny];
-    for (int nx=0; nx<Nx; ++nx)
-      for (int nz=0; nz<Nz; ++nz) 
-	q(nx,ny,nz,0) += 0.5*(square(u(nx,ny,nz,0)) + //  + Uny) + 
-			      square(u(nx,ny,nz,1)) + 
-			      square(u(nx,ny,nz,2))); 
-  }
-  u.makeState(uxzstate, uystate); 
-  q.makeState(p.xzstate(), p.ystate()); 
-}
-
 void changeBaseFlow(const ChebyCoeff& ubase0, const FlowField& ufluc0, 
 		const FlowField& q0arg, 
 		const ChebyCoeff& ubase1, FlowField& u1, FlowField& q1){
@@ -908,16 +834,18 @@ void changeBaseFlow(const ChebyCoeff& ubase0, const FlowField& ufluc0,
 }
   
 DNSFlags::DNSFlags(MeanConstraint  constraint_,
-		     TimeStepMethod  timestepping_,
-		     NonlinearMethod nonlinearity_,
-		     Dealiasing      dealiasing_,
-		     Verbosity       verbosity_)
+		   TimeStepMethod  timestepping_,
+		   NonlinearMethod nonlinearity_,
+		   Dealiasing      dealiasing_,
+		   bool taucorrection_,
+		   Verbosity       verbosity_)
   :
   constraint(constraint_),
   timestepping(timestepping_),
   nonlinearity(nonlinearity_),
   dealiasing(dealiasing_),
-  verbosity(verbosity_)
+  verbosity(verbosity_),
+  taucorrection(taucorrection)
 {
   if (dealias_y() && (nonlinearity != Rotational)) {
     cerr << "DNSFlags::DNSFlags: DealiasY and DealiasXYZ work only with\n";
